import { DateTime, Info } from 'luxon';
import _ from 'lodash';
import React, { useEffect, useState, useRef } from 'react';
import { Table, Card, Menu, Icon, Header, Ref, List } from 'semantic-ui-react';

var useClickOut = function (ref, onClickOut) {
    var isInDocument = function (element) { return document.contains(element); };
    var isInRef = function (element) { var _a; return (_a = ref.current) === null || _a === void 0 ? void 0 : _a.contains(element); };
    var onClickOutside = function (e) {
        var element = e.target;
        if (ref.current !== null && isInDocument(element) && !isInRef(element)) {
            e.preventDefault();
            e.stopPropagation();
            onClickOut();
        }
    };
    var addListener = function () { return document.addEventListener('click', onClickOutside); };
    var removeListener = function () { return document.removeEventListener('click', onClickOutside); };
    useEffect(function () {
        if (ref.current)
            addListener();
        return removeListener;
    }, [ref]);
    // return ref;
};

var Calendar = function (_a) {
    var _b = _a.datetime, datetime = _b === void 0 ? DateTime.now() : _b, setDatetime = _a.setDatetime, onSet = _a.onSet;
    var set = function (values) {
        if (setDatetime)
            setDatetime(datetime.set(values));
        if (onSet)
            onSet();
    };
    return (React.createElement(Table, { celled: true, className: 'unstackable' },
        React.createElement(Table.Header, null,
            React.createElement(Table.Row, null, sundayFirstWeekdays('short').map(function (d) { return React.createElement(Table.HeaderCell, { key: d, content: React.createElement("div", { className: 'sizer' }, d) }); }))),
        React.createElement(Table.Body, null, calDaysChunks(datetime).map(function (week, w) {
            return React.createElement(Table.Row, { key: w }, week.map(function (dt, d) {
                return React.createElement(Table.Cell, { active: datetime.day === dt.day && datetime.month === dt.month, className: datetime.month !== dt.month ? 'otherMonth' : '', onClick: function () { return set(dt); }, key: d, content: React.createElement("div", { className: 'sizer' }, dt.day) });
            }));
        }))));
};

var Months = function (_a) {
    var _b = _a.datetime, datetime = _b === void 0 ? DateTime.now() : _b, setDatetime = _a.setDatetime, onSet = _a.onSet;
    var set = function (values) {
        if (setDatetime)
            setDatetime(datetime.set(values));
        if (onSet)
            onSet();
    };
    var months = Info.months('short');
    return (React.createElement(Card.Group, { className: "compact months", itemsPerRow: 4 }, months.map(function (month, m) {
        return React.createElement(Card, { key: m, className: datetime.month === m + 1 ? 'active' : '', content: month, onClick: function () { return set({ month: m + 1 }); } });
    })));
};

var Years = function (_a) {
    var _b = _a.datetime, datetime = _b === void 0 ? DateTime.now() : _b, setDatetime = _a.setDatetime, onSet = _a.onSet;
    var set = function (values) {
        if (setDatetime)
            setDatetime(datetime.set(values));
        if (onSet)
            onSet();
    };
    var getYears = function (dt, past, present) {
        if (past === void 0) { past = 6; }
        if (present === void 0) { present = 6; }
        var year = dt.year;
        var arr = [];
        for (var y = year - past; y < year + present; y++) {
            arr.push(y);
        }
        return arr;
    };
    var _c = useState(getYears(datetime)), years = _c[0], setYears = _c[1];
    var _d = useState(6), past = _d[0], setPast = _d[1];
    var _e = useState(6), future = _e[0], setFuture = _e[1];
    useEffect(function () {
        setYears(getYears(datetime, past, future));
    }, [past, future, setYears]);
    return (React.createElement(React.Fragment, null,
        React.createElement(Menu, { secondary: true },
            React.createElement(Menu.Item, { icon: true, onClick: function () { return setPast(past + 24); }, content: React.createElement(Icon, { name: 'angle double left' }) }),
            React.createElement(Menu.Item, { icon: true, onClick: function () { return setPast(past + 4); }, content: React.createElement(Icon, { name: 'angle left' }) }),
            React.createElement(Header, { className: "half-opacity" },
                years[0],
                " - ",
                years[years.length - 1]),
            React.createElement(Menu.Item, { icon: true, onClick: function () { return setFuture(future + 4); }, content: React.createElement(Icon, { name: 'angle right' }) }),
            React.createElement(Menu.Item, { icon: true, onClick: function () { return setFuture(future + 24); }, content: React.createElement(Icon, { name: 'angle double right' }) })),
        React.createElement(Card.Group, { className: "compact years", itemsPerRow: 4 }, years.map(function (year) {
            return React.createElement(Card, { key: year, className: datetime.year === year ? 'active' : '', content: year, onClick: function () { return set({ year: year }); } });
        }))));
};

var Date = function (_a) {
    var _b = _a.datetime, datetime = _b === void 0 ? DateTime.now() : _b, setDatetime = _a.setDatetime; _a.onSet; var onClickOut = _a.onClickOut;
    var _c = useState(false), showingMonths = _c[0], setShowingMonths = _c[1];
    var _d = useState(false), showingYears = _d[0], setShowingYears = _d[1];
    var prevMonth = function () { return setDatetime ? setDatetime(datetime.minus({ month: 1 })) : null; };
    var nextMonth = function () { return setDatetime ? setDatetime(datetime.plus({ month: 1 })) : null; };
    var clickRef = useRef(null);
    useClickOut(clickRef, onClickOut);
    return (React.createElement(Ref, { innerRef: clickRef },
        React.createElement(Card, { className: "picker" },
            React.createElement(Card.Content, null,
                React.createElement(Menu, { secondary: true },
                    React.createElement(Menu.Item, { icon: true, onClick: prevMonth, content: React.createElement(Icon, { name: 'arrow left' }) }),
                    React.createElement(Header, null,
                        React.createElement("span", { className: 'clickable', onClick: function () { return setShowingMonths(true); } }, datetime.monthLong),
                        React.createElement("span", { className: 'clickable', onClick: function () { return setShowingYears(true); } }, datetime.year)),
                    React.createElement(Menu.Item, { icon: true, onClick: nextMonth, content: React.createElement(Icon, { name: 'arrow right' }) })),
                !showingMonths && !showingYears &&
                    React.createElement(Calendar, { datetime: datetime, setDatetime: setDatetime, onSet: onClickOut }),
                showingMonths &&
                    React.createElement(Months, { datetime: datetime, setDatetime: setDatetime, onSet: function () { setShowingMonths(false); } }),
                showingYears &&
                    React.createElement(Years, { datetime: datetime, setDatetime: setDatetime, onSet: function () { return setShowingYears(false); } })))));
};

var DatePicker = function (_a) {
    var _b = _a.datetime, datetime = _b === void 0 ? DateTime.now() : _b, setDatetime = _a.setDatetime; _a.onSet;
    var _c = useState(false), showingPicker = _c[0], setShowingPicker = _c[1];
    var showPicker = function () { return setShowingPicker(true); };
    var hidePicker = function () { return setShowingPicker(false); };
    return (React.createElement("div", { className: 'datepicker container' },
        React.createElement("div", { className: "clickable", onClick: showPicker }, datetime.toLocaleString()),
        showingPicker &&
            React.createElement(Date, { datetime: datetime, setDatetime: setDatetime, onClickOut: hidePicker })));
};

/******************************************************************************
Copyright (c) Microsoft Corporation.

Permission to use, copy, modify, and/or distribute this software for any
purpose with or without fee is hereby granted.

THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES WITH
REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY
AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY SPECIAL, DIRECT,
INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM
LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR
OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR
PERFORMANCE OF THIS SOFTWARE.
***************************************************************************** */

function __spreadArray(to, from, pack) {
    if (pack || arguments.length === 2) for (var i = 0, l = from.length, ar; i < l; i++) {
        if (ar || !(i in from)) {
            if (!ar) ar = Array.prototype.slice.call(from, 0, i);
            ar[i] = from[i];
        }
    }
    return to.concat(ar || Array.prototype.slice.call(from));
}

var useScrollToActive = function () {
    var deps = [];
    for (var _i = 0; _i < arguments.length; _i++) {
        deps[_i] = arguments[_i];
    }
    var scrollRef = useRef(null);
    useEffect(function () {
        if (scrollRef.current) {
            var container = scrollRef.current;
            var activeItem = container.querySelector('.item.active');
            if (activeItem) {
                var _a = activeItem.getBoundingClientRect(), itemTop = _a.top; _a.bottom;
                var _b = container.getBoundingClientRect(), containerTop = _b.top, containerBottom = _b.bottom; _b.height;
                var middleYPos = (containerBottom - containerTop) / 2;
                var distanceToItem = itemTop - middleYPos;
                container.scrollTop = container.scrollTop + distanceToItem;
            }
        }
    }, __spreadArray(__spreadArray([], deps, true), [scrollRef.current], false));
    return scrollRef;
};

var Hours = function (_a) {
    var _b = _a.datetime, datetime = _b === void 0 ? DateTime.now() : _b, setDatetime = _a.setDatetime, onSet = _a.onSet, onClickOut = _a.onClickOut;
    var set = function (values) {
        if (setDatetime)
            setDatetime(datetime.set(values));
        if (onSet)
            onSet();
    };
    var hours = [0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 18, 19, 20, 21, 22, 23];
    var scrollRef = useScrollToActive(datetime);
    useClickOut(scrollRef, onClickOut);
    return (React.createElement(Ref, { innerRef: scrollRef },
        React.createElement(List, { className: "scrolling picker" }, hours.map(function (hour, h) {
            return React.createElement(List.Item, { key: h, className: datetime.hour === h ? 'active' : '', content: datetime.set({ hour: hour }).toLocaleString({ hour: 'numeric' }), onClick: function () { return set({ hour: h }); } });
        }))));
};

var Minutes = function (_a) {
    var _b = _a.datetime, datetime = _b === void 0 ? DateTime.now() : _b, setDatetime = _a.setDatetime, onSet = _a.onSet, onClickOut = _a.onClickOut;
    var set = function (values) {
        if (setDatetime)
            setDatetime(datetime.set(values));
        if (onSet)
            onSet();
    };
    var minutes = [0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 18, 19, 20, 21, 22, 23, 24, 25, 26, 27, 28, 29, 30, 31, 32, 33, 34, 35, 36, 37, 38, 39, 40, 41, 42, 43, 44, 45, 46, 47, 48, 49, 50, 51, 52, 53, 54, 55, 56, 57, 58, 59];
    var scrollRef = useScrollToActive(datetime);
    useClickOut(scrollRef, onClickOut);
    return (React.createElement(Ref, { innerRef: scrollRef },
        React.createElement(List, { className: "scrolling picker" }, minutes.map(function (minute, m) {
            return React.createElement(List.Item, { key: m, className: datetime.minute === m ? 'active' : '', content: datetime.set({ minute: minute }).toFormat('h:mm'), onClick: function () { return set({ minute: m }); } });
        }))));
};

var TimePicker = function (_a) {
    var _b = _a.datetime, datetime = _b === void 0 ? DateTime.now() : _b, setDatetime = _a.setDatetime; _a.onSet;
    var _c = useState(false), showingHours = _c[0], setShowingHours = _c[1];
    var _d = useState(false), showingMinutes = _d[0], setShowingMinutes = _d[1];
    return (React.createElement("div", { className: 'timepicker' },
        React.createElement("span", { className: 'clickable', onClick: function () { return setShowingHours(true); } }, datetime.toFormat('h')),
        ":",
        React.createElement("span", { className: 'clickable compact', onClick: function () { return setShowingMinutes(true); } }, datetime.toFormat('mm')),
        React.createElement("span", { className: 'clickable', onClick: function () { return setShowingHours(true); } }, datetime.toFormat('a')),
        showingHours &&
            React.createElement(Hours, { datetime: datetime, setDatetime: setDatetime, onSet: function () { return setShowingHours(false); }, onClickOut: function () { return setShowingHours(false); } }),
        showingMinutes &&
            React.createElement(Minutes, { datetime: datetime, setDatetime: setDatetime, onSet: function () { return setShowingMinutes(false); }, onClickOut: function () { return setShowingMinutes(false); } })));
};

var DateTimePicker = function (_a) {
    var initial = _a.initial, _b = _a.selectDate, selectDate = _b === void 0 ? true : _b, _c = _a.selectTime, selectTime = _c === void 0 ? true : _c, _d = _a.name, name = _d === void 0 ? 'datetime' : _d;
    var _e = useState(initial || DateTime.now()), datetime = _e[0], setDatetime = _e[1];
    return (React.createElement("div", { className: 'datetimepicker' },
        React.createElement("input", { hidden: true, name: name, value: datetime.toMillis(), readOnly: true }),
        React.createElement("input", { hidden: true, name: name + '-date-only', value: datetime.toISODate(), readOnly: true }),
        React.createElement("input", { hidden: true, name: name + '-time-only', value: datetime.toISOTime(), readOnly: true }),
        selectDate &&
            React.createElement(DatePicker, { datetime: datetime, setDatetime: setDatetime }),
        selectTime &&
            React.createElement(TimePicker, { datetime: datetime, setDatetime: setDatetime })));
};

function sundayFirstWeekdays(length, options) {
    var weekdays = Info.weekdays(length, options);
    var finalDay = weekdays.pop();
    weekdays.splice(0, 0, finalDay);
    return weekdays;
}
function daysInMonthArray(dt) {
    var arr = [];
    for (var i = 1; i < dt.daysInMonth + 1; i++) {
        arr.push(i);
    }
    return arr;
}
function firstDayOffset(dt) {
    // FOR CALENDARS STARTING ON SUNDAY
    var firstDay = dt.set({ day: 1 });
    var weekday = firstDay.weekday;
    return weekday % 7;
}
function calDaysChunks(dt) {
    var getDateObject = function (lux) { return ({ month: lux.month, day: lux.day }); };
    var days = daysInMonthArray(dt).map(function (day) { return getDateObject(dt.set({ day: day })); });
    var offset = firstDayOffset(dt);
    var prev = dt.set({ day: 0 }).minus({ day: 1 });
    var next = dt.set({ day: 1 }).plus({ month: 1 });
    while (offset > 0) {
        days.splice(0, 0, getDateObject(prev));
        prev = prev.minus({ day: 1 });
        offset--;
    }
    var chunks = _.chunk(days, 7);
    while (chunks[chunks.length - 1].length < 7) {
        chunks[chunks.length - 1].push(getDateObject(next));
        next = next.plus({ day: 1 });
    }
    return chunks;
}

export { DateTimePicker, calDaysChunks, daysInMonthArray, firstDayOffset, sundayFirstWeekdays };
//# sourceMappingURL=index.js.map
